﻿using System;
using System.Net;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.Generic;
using SL_ACO;
using System.IO;
using System.Threading;

namespace SL_ACO.ACO
{
    #region enum

    public enum tipProblema
    {
        TSP = 0,
        VRP = 1,
        error = 2,
    }

    #endregion

    public class AntColony
    {
        #region Fields

        #region Internal

        //internal varijable koje mravi koriste za pretragu
        internal float[,] Trail;      //matrica Traila - trag feromona na stazi i,j (& j,i)
        internal Random randomizer;   //klasa za randomizaciju odluka mrava i ulaza
        internal Grad[] gradovi;      //niz gradova, indeks u nizu je redni broj grada (n+nulti član)

        #endregion

        #region Private

        //parametri optimizacije
        private int brojMrava;       //broj mrava u koloniji
        private int brojVozila;      //broj vozila dozvoljenih u VRP-u
        private float alpha;         //alpha - važnost Trail-a
        private float beta;          //beta - važnost vidljivosti
        private float rho;           //rho - faktor isparavanja
        private int NC_max;          //NC - broj ciklusa

        //parametri problema
        private int brojGradova;             //broj gradova u problemu
        private tipProblema TipProblema;     //Enum tip problema VRP(1) ili TSP(0)
        private int RedZaPremade;

        //varijable za tok programa i zapisivanje nađenog rješenja, private
        private Panel DrawArea;     //panel za crtanje gradova i najbolje rute
        private Panel GrafArea;     //panel za crtanje grafa ruta u ciklusima
        private Page Strana;        // stranica s kojom komunicira kolonija

        //graficke vrijednosti
        private double faktorSirine;         //faktor s kojim se množe x koordinate gradova pri crtanju
        private int sirinaDraw;             //predstavlja granicu kolika je draw area (550)
        private double faktorVisine;         //faktor s kojim se mnozi y koordinata gradova pri crtanju
        private int visinaDraw;             //predstavlja granicu kolika je draw area (550)

        //varijable koje se resetiraju gumbom reset
        //vremenske varijable za korake i cikluse
        private int time_step;          //brojac stepa
        private int time_ciklus;        //brojac ciklusa
        //rezultati nađenih ruta
        private double BestRoute;        //duljina najbolje rute
        private int BestFoundAt;        //ciklus u kojem je nadjena najbolja ruta
        private int[] BestRouteCitys;   //poredani gradovi kojima je isao najoptimalniji mrav
        private double[,] BestRouteGraf;   //matrica vrijednosti, zapisuje svaki novi best route i u kojem je ciklusu nadjen
        private int BestFoundPointer;   //pokazivac na red u matrici (koliko je najboljih ruta do sad bilo)
        //vremenske varijable, za praćenje vremena u step funkciji
        private DateTime PocetakRada;   //pocetak rada algoritma (pritisak na gumb)
        private DateTime KrajOpt;       //zavrsetak optimizacije
        private Boolean FirstStep;      //da li je ovo PRVI korak ('da' samo nakon reseta i loadanja)

        //MRAVI
        private List<Mrav> mravi;
        
        //singleton -instance- obrazac
        private static AntColony instance = null;

        #endregion

        #endregion

        #region Methods

        #region Initial

        /// <summary>
        /// postavlja parametre na početne vrijednosti
        /// </summary>
        private void pocetneVrijednosti()
        {
            this.brojVozila = 4;
            this.RedZaPremade = 0;
            this.faktorSirine = 1;
            this.sirinaDraw = 550;
            this.faktorVisine = 1;
            this.visinaDraw = 550;
            this.randomizer = new Random();
            this.mravi = new List<Mrav>();
            this.BestRouteGraf = new double[1000,2];
            this.BestFoundPointer = 0;
            this.FirstStep = true;
            this.alpha = 1;
            this.beta = 1;
            this.PocetakRada = DateTime.Now;
            this.KrajOpt = DateTime.Now;
            this.rho = (float)0.7;
            this.NC_max = 100;
            this.time_ciklus = 0;
            this.time_step = 0;
            this.BestRoute = 10000000000;
            this.BestFoundAt = 0;
            this.Trail = new float[300, 300];
            this.BestRouteCitys = new int[600];
            int i, j;
            for (i = 0; i < 300; i++)
            {
                for (j = 0; j < 300; j++)
                {
                    this.Trail[i, j] = 10;
                };
            };

        }

        /// <summary>
        /// konstruktor, poziva pocetne vrijednosti
        /// </summary>
        private AntColony()
        {
            this.pocetneVrijednosti();
        }

        /// <summary>
        /// Page se pri inicijalizaciji predstavi kao Page, i posalje svoje
        /// dijelove: 2 panela za iscrtavanje rezultata.
        /// </summary>
        /// <param name="stranica">Page Vlasnik kolonije</param>
        /// <param name="graf">Panel za crtanje grafa duljine najboljih ruta</param>
        /// <param name="InitdrawArea">Panel za crtanje gradova i najbolje pronađene rute</param>
        public void JaviPage(Page stranica, Panel graf, Panel InitdrawArea)
        {
            this.Strana = stranica;
            this.DrawArea = InitdrawArea;
            this.GrafArea = graf;
            return;
        }
        
        #endregion

        #region Resets

        /// <summary>
        /// zaustavlja optimizaciju i vraća sve vremenske parametre na početak
        /// </summary>
        public void Reset()
        {
            this.time_ciklus = 0;
            this.time_step = 0;
            this.BestRoute = 1000000000;
            this.BestFoundPointer = 0;
            this.BestFoundAt = 0;
            this.InitDraw();
            this.PocetakRada = DateTime.Now;
            this.KrajOpt = DateTime.Now;
            this.FirstStep = true;
            //reset Trail na c=10
            int i=0;
            int j=0;
            for (i = 0; i < 300; i++)
            {
                for (j = 0; j < 300; j++)
                {
                    this.Trail[i, j] = 10;
                };
            };
            //reset Najbolja ruta
            for (i = 0; i < 600; i++)
            {
                this.BestRouteCitys[i] = 0;
            };
            //reset grafnajboljih ruta
            for (i = 0; i < 1000; i++)
            {
                this.BestRouteGraf[i, 0] = 100000000;
                this.BestRouteGraf[i, 1] = 0;
            };
        }

        /// <summary>
        /// poziva Cleanup funkciju svih mrava
        /// </summary>
        private void ResetirajMrave()
        {
            foreach (Mrav k in mravi)
            {
                k.Cleanup();
            }
            return;
        }

        #endregion

        #region Sets

        /// <summary>
        /// postavlja broj vozila u VRP-u
        /// </summary>
        /// <param name="noviBrojVozila">broj vozila za VRP</param>
        public void Set_vozila(int noviBrojVozila)
        {
            if ((noviBrojVozila > 0) && (noviBrojVozila < 100))
            {
                this.brojVozila = noviBrojVozila;
            }
            return;
        }

        /// <summary>
        /// postavlja m - broj mrava, ukoliko je argument valjan (izmedju 1 i 1000)
        /// </summary>
        /// <param name="m">novi m</param>
        public void Set_m(int m)
        {
            if ((m > 0) && (m < 1000))
            {
                this.brojMrava = m;
            };
        }

        /// <summary>
        /// postavlja alpha - važnost traga, ukoliko je argument valjan (0 <= a < 100)
        /// </summary>
        /// <param name="a">novi alpha</param>
        public void Set_alpha(float a)
        {
            if ((a >= 0) && (a < 100))
            {
                this.alpha = a;
            };
        }

        /// <summary>
        /// postavlja beta - važnost udaljenosti, ukoliko je valjana (0 <= b < 100)
        /// </summary>
        /// <param name="b">novi beta</param>
        public void Set_beta(float b)
        {
            if ((b >= 0) && (b < 100))
            {
                this.beta = b;
            };
        }

        /// <summary>
        /// postavlja NC - broj ciklusa, ukoliko je roj valjan (izmedju 1 i 20000)
        /// </summary>
        /// <param name="NC">novi NC</param>
        public void Set_NC(int NC)
        {
            if ((NC > 1) && (NC < 20000))
            {
                this.NC_max = NC;
            };
        }

        /// <summary>
        /// postavlja rho - faktor isparavanja, ukoliko je valjan (izmedju 0 i 1)
        /// </summary>
        /// <param name="r">novi rho</param>
        public void Set_rho(float r)
        {
            if ((r >= 0) && (r <= 1))
            {
                this.rho = r;
            };
        }

        /// <summary>
        /// postavlja Tip problema, 0=TSP  1=VRP
        /// </summary>
        /// <param name="tip">int koji predstavlja tip problema</param>
        public void Set_Tip(tipProblema tip)
        {
            if ((tip == tipProblema.TSP) || (tip == tipProblema.VRP))
            {
                this.TipProblema = tip;
            };
            return;
        }

        #endregion

        #region Gets

        /// <summary>
        /// zatraži broj vozila u VRP-u
        /// </summary>
        /// <returns>broj vozila dozvoljeno</returns>
        public int Get_brojVozila()
        {
            return this.brojVozila;
        }

        /// <summary>
        /// zatraži tip trenutno učitanog problema
        /// </summary>
        /// <returns>tip problema</returns>
        public tipProblema Get_tip()
        {
            return this.TipProblema;
        }

        /// <summary>
        /// zatraži n - broj gradova
        /// </summary>
        /// <returns>broj gradova</returns>
        public int Get_n()
        {
            return this.brojGradova;
        }

        /// <summary>
        /// zatraži m -broj mrava
        /// </summary>
        /// <returns>broj mrava</returns>
        public int Get_m()
        {
            return this.brojMrava;
        }

        /// <summary>
        /// zatraži alpha - značaj Traila
        /// </summary>
        /// <returns>alpha</returns>
        public float Get_alpha()
        {
            return this.alpha;
        }

        /// <summary>
        /// zatraži beta - značaj udaljenosti
        /// </summary>
        /// <returns>beta</returns>
        public float Get_beta()
        {
            return this.beta;
        }

        /// <summary>
        /// zatraži NC_max - broj ciklusa koji će se izvoditi
        /// </summary>
        /// <returns>NC</returns>
        public int Get_NC()
        {
            return this.NC_max;
        }

        /// <summary>
        /// zatraži rho - koeficijent isparavanja traga
        /// </summary>
        /// <returns>rho</returns>
        public float Get_rho()
        {
            return this.rho;
        }

        #endregion

        #region InputOutput Problem

        /// <summary>
        /// dodaje jedan grad u listu, pozvana od click eventa sa canvasa
        /// </summary>
        /// <param name="X">X koordinata grada</param>
        /// <param name="Y">Y koordinata grada</param>
        public void AddCity(float X, float Y)
        {
            this.brojGradova++;
            this.gradovi[this.brojGradova] = new Grad(X/faktorSirine, Y/faktorVisine);
            this.Reset();
            return;
        }

        /// <summary>
        /// Ucitava novi problem iz klase Ulaz
        /// </summary>
        /// <param name="area">Area za crtanje gradova i rute</param>
        /// <param name="varijanta">1=random, 2=clean, 3=load, 4 = stored</param>
        /// <param name="graf">Panel za crtanje grafa najbolje rute</param>
        public void Read(int varijanta)
        {
            //po case(varijanta) radi zadano
            #region Ucitaj Problem varijante
            switch (varijanta)
            {
                case 1: //RANDOM
                    Ulaz ulazniGradovi1 = new Ulaz(0);
                    this.brojGradova = ulazniGradovi1.brojGradova;
                    this.gradovi = new Grad[300];
                    this.gradovi[0] = new Grad(0, 0);
                    ulazniGradovi1.lista.CopyTo(gradovi, 1);
                    this.TipProblema = ulazniGradovi1.Tip;
                    this.brojVozila = ulazniGradovi1.brojVozila;
                    break;
                case 2: //CLEAN
                    this.gradovi = new Grad[300];
                    this.gradovi[0] = new Grad(0, 0);
                    this.TipProblema = 0;
                    this.brojGradova = 0;
                    this.brojMrava = 10;
                    this.brojVozila = 4;
                    break;
                case 3: //LOAD
                    Ulaz ulazniGradovi3 = new Ulaz(1);
                    if (ulazniGradovi3.Tip == tipProblema.error)
                    {
                        return;
                    }
                    this.brojGradova = ulazniGradovi3.brojGradova;
                    this.gradovi = new Grad[300];
                    this.gradovi[0] = new Grad(0, 0);
                    ulazniGradovi3.lista.CopyTo(gradovi, 1);
                    this.TipProblema = ulazniGradovi3.Tip;
                    this.brojVozila = ulazniGradovi3.brojVozila;
                    break;
                case 4: //STORED
                    Ulaz ulazniGradovi4 = new Ulaz(2 + this.RedZaPremade); // pozove neki problem od zadanih
                    this.RedZaPremade++;
                    this.brojGradova = ulazniGradovi4.brojGradova;
                    this.gradovi = new Grad[300];
                    this.gradovi[0] = new Grad(0, 0);
                    ulazniGradovi4.lista.CopyTo(gradovi, 1);
                    this.TipProblema = ulazniGradovi4.Tip;
                    this.brojVozila = ulazniGradovi4.brojVozila;
                    break;
            }//end case
            #endregion
            //ucitavanje granica draw area, zbog gradova > 1
            #region nastimavanje faktora za Panel Draw Area
            this.faktorSirine = (double)this.sirinaDraw*0.98;
            this.faktorVisine = (double)this.visinaDraw*0.98;
            if (this.brojGradova > 0)
            {
                double maxVisina = 0;
                double maxSirina = 0;
                foreach (Grad g in this.gradovi)
                {
                    if (g != null)
                    {
                        if (g.get_x() > maxSirina)
                        {
                            maxSirina = g.get_x();
                        }
                        if (g.get_y() > maxVisina)
                        {
                            maxVisina = g.get_y();
                        }
                    }
                }//end foreach
                if (maxSirina > 1)
                {
                    this.faktorSirine = this.sirinaDraw / (maxSirina * 1.06);
                }
                if (maxVisina > 1)
                {
                    this.faktorVisine = this.visinaDraw / (maxVisina * 1.06);
                }
            }//end if
            #endregion
            this.Reset();
            return;
        }

        /// <summary>
        /// poziva sejvanje trenutno aktivnog problema
        /// </summary>
        public void Save(string path)
        {
            if (this.brojGradova > 0) //ako ima što za sejvat, onda sejvaj
            {
                Izlaz sejvanje = new Izlaz(path);
            }
            return;
        }

        #endregion

        #region Algoritam
        
        /// <summary>
        /// izracuna da li su mravi nasli najbolju rutu u trenutnom ciklusu,
        /// ako da, onda ju apise na odgovarajuci nacin
        /// </summary>
        private void CalculateHandleBestRoute()
        {
            double duljina = 0;
            foreach (Mrav k in mravi)
            {
                //racuna najbolju rutu za svakog mrava
                duljina = k.DuljinaPuta();
                if (duljina < (this.BestRoute * 0.99999)) //faktor za izbjegavanje double & float pogreske
                {
                    this.BestRoute = duljina;
                    this.BestFoundAt = this.time_ciklus;
                    this.BestRouteGraf[this.BestFoundPointer, 0] = duljina;
                    this.BestRouteGraf[this.BestFoundPointer, 1] = this.BestFoundAt;
                    this.BestFoundPointer++;
                    for (int brojac = 0; brojac < k.GradovaJeProsao + 1; brojac++)
                    {
                        this.BestRouteCitys[brojac] = k.Tabu_lista[brojac];
                    }
                    //daj informacije stranici kad nađeš novi optimum
                    this.Strana.Novosti(this.time_ciklus, this.BestRoute, this.BestFoundAt, (DateTime.Now - PocetakRada).ToString());
                    this.Sketch();
                };
            }
            return;
        }
                
        /// <summary>
        /// stvara odgovarajući broj mrava za Step i Start,
        /// prema parametrima broj mrava i tip problema (pocetni grad mrava)
        /// </summary>
        private void PostaviMrave()
        {
            this.mravi.Clear();
            while (this.mravi.Count != this.brojMrava)
            {
                if (this.TipProblema == 0)
                {
                    this.mravi.Add(new Mrav(this.brojGradova, 1 + this.randomizer.Next(this.brojGradova)));
                }
                else
                {
                    this.mravi.Add(new Mrav(this.brojGradova, 1));
                }

            };
            return;
        }

        /// <summary>
        /// poziva n+1 koraka mrava (+1 za slucaj VRP sa 1 agentom)
        /// </summary>
        private void OdradiCiklus()
        {
            this.time_step = 0;
            for (this.time_step = 0; this.time_step < this.brojGradova + 2; this.time_step++)
            {
                foreach (Mrav k in mravi) //svakom mravu
                {
                    k.nextStep();  //poziv koraka mrava
                }
            }
            return;
        }

        /// <summary>
        /// smanjuje vrijednosti u cijeloj matrici Trail prema faktoru isparavanja rho
        /// </summary>
        private void IspariTrag()
        {
            int i = 0;
            int j = 0;
            for (i = 1; i < this.brojGradova + 1; i++)
            {
                for (j = 1; j < this.brojGradova + 1; j++)
                {
                    this.Trail[i, j] = this.rho * this.Trail[i, j];
                    if (this.Trail[i, j] == 0) //izbjegavanje errora u računu next stepa mrava
                    {
                        this.Trail[i, j] = 1;
                    }
                };
            };
            return;
        }

        /// <summary>
        /// pokreće optimizaciju (najprije stvara potrebne mrave)
        /// </summary>
        public void Start()
        {
            DateTime pocetakOpt = DateTime.Now;
            //brisanje mrava te stvaranje novih
            if (FirstStep == true)
            {
                PocetakRada = DateTime.Now;
                KrajOpt = DateTime.Now;
                this.PostaviMrave();
                this.FirstStep = false;
            }
            else
            {
                PocetakRada = PocetakRada + (pocetakOpt - KrajOpt);
            };

            //ACO ALGORITAM
            while (this.time_ciklus < this.NC_max)
            {
                this.time_ciklus++;
                //koraci mrava za jedan ciklus
                this.OdradiCiklus();

                //isparavanje traga
                this.IspariTrag();

                //izracun najbolje rute
                this.CalculateHandleBestRoute();

                //resetiranje mrava
                this.ResetirajMrave();
            }

            KrajOpt = DateTime.Now;
            //daj informacije stranici kaj završi optimizacija
            this.Strana.Novosti(this.time_ciklus, this.BestRoute, this.BestFoundAt, (KrajOpt - PocetakRada).ToString());
            this.Sketch();
        }

        /// <summary>
        /// pokrece optimizaciju za jedan korak, ukoliko je prvi korak stvori mrave
        /// </summary>
        public void Step()
        {
            DateTime pocetakOpt = DateTime.Now;
            if (FirstStep == true)
            {
                PocetakRada = DateTime.Now;
                KrajOpt = DateTime.Now;
                this.PostaviMrave();
                this.FirstStep = false;
            }
            else
            {
                PocetakRada = PocetakRada + (pocetakOpt - KrajOpt);
            };

            //STEP-ast ACO
            int StepChecker = this.BestFoundPointer;
            while ((StepChecker == this.BestFoundPointer) && (this.time_ciklus < this.NC_max))
            {
                this.time_ciklus++;
                //koraci mrava za jedan ciklus
                this.OdradiCiklus();

                //isparavanje traga
                this.IspariTrag();

                //izracun najbolje rute
                this.CalculateHandleBestRoute();

                //resetiranje mrava
                this.ResetirajMrave();
            }

            KrajOpt = DateTime.Now;
            //daj informacije stranici kaj završi optimizacija
            this.Strana.Novosti(this.time_ciklus, this.BestRoute, this.BestFoundAt, (KrajOpt - PocetakRada).ToString());
            this.Sketch();
        }

        #endregion

        #region Grafika

        /// <summary>
        /// crta nabolju nađenu rutu na graf
        /// </summary>
        private void Sketch()
        {
            this.InitDraw();
            this.GrafRouta();
            switch (this.TipProblema)
            {
                case tipProblema.TSP:
                    #region TSP linija
                    Point tocka = new Point();
                    Polyline linija = new Polyline();
                    for (int i = 0; i < 599; i++)
                    {
                        if (this.BestRouteCitys[i] != 0)
                        {
                            tocka.X =2+ gradovi[this.BestRouteCitys[i]].get_x() * this.faktorSirine;
                            tocka.Y =2+ gradovi[this.BestRouteCitys[i]].get_y() * this.faktorVisine;
                            if ((tocka.X != 0) || (tocka.Y != 0))
                            {
                                linija.Points.Add(tocka);
                            };
                        };
                    };
                    linija.Stroke = new SolidColorBrush(Colors.Red);
                    linija.StrokeThickness = 1;
                    linija.Opacity = 1;
                    this.DrawArea.Children.Add(linija);
                    break;
                    #endregion
                case tipProblema.VRP:
                    #region VRP linije
                    Color[] Boje = new Color[8];
                    Boje[0] = Colors.Red;
                    Boje[1] = Colors.Purple;
                    Boje[2] = Colors.Black;
                    Boje[3] = Colors.Brown;
                    Boje[4] = Colors.Green;
                    Boje[5] = Colors.Orange;
                    Boje[6] = Colors.Gray;
                    Boje[7] = Colors.Yellow;
                    Point pocetna = new Point();
                    pocetna.X =2+ this.gradovi[this.BestRouteCitys[0]].get_x() * this.faktorSirine;
                    pocetna.Y =2+ this.gradovi[this.BestRouteCitys[0]].get_y() * this.faktorVisine;
                    Polyline[] Linije = new Polyline[this.brojVozila + 2];
                    for (int brojac = 0; brojac < this.brojVozila + 2; brojac++)
                    {
                        Linije[brojac] = new Polyline();
                    }
                    int rednibroj = 0;
                    Point tockica = new Point();
                    for (int i = 0; i < 599; i++)
                    {
                        tockica.X =2+ this.gradovi[this.BestRouteCitys[i]].get_x() * this.faktorSirine;
                        tockica.Y =2+ this.gradovi[this.BestRouteCitys[i]].get_y() * this.faktorVisine;
                        Linije[rednibroj].Points.Add(tockica);
                        if ((tockica == pocetna) && (Linije[rednibroj].Points.Count != 1))
                        {
                            rednibroj++;
                            Linije[rednibroj].Points.Add(pocetna);
                        }
                    }
                    int Boja = 0;
                    for (int j = 0; j < rednibroj; j++)
                    {
                        Boja = j;
                        while (Boja > 7)
                        {
                            Boja -= 8;
                        }
                        Linije[j].Stroke = new SolidColorBrush(Boje[Boja]);
                        Linije[j].StrokeThickness = 1;
                        Linije[j].Opacity = 1;
                        this.DrawArea.Children.Add(Linije[j]);
                    }
                    break;
                    #endregion
                default:
                    return;
            }
            return;
        }

        /// <summary>
        /// crta najbolje route na graf
        /// </summary>
        private void GrafRouta()
        {
            int i;
            Line optimum = new Line();
            Polyline GrafRouta = new Polyline();
            Point tocka = new Point(0, 0);
            GrafRouta.Stroke = new SolidColorBrush(Colors.Red);
            GrafRouta.StrokeThickness = 2;
            GrafRouta.Opacity = 1;
            double visinaFaktor = 170.0 / BestRouteGraf[0, 0];
            double sirinaFaktor = 680.0 / this.NC_max;
            for (i = 0; i < this.BestFoundPointer; i++)
            {
                tocka.Y = 180 - (BestRouteGraf[i, 0] * visinaFaktor);
                tocka.X = 50 + BestRouteGraf[i, 1] * sirinaFaktor;
                GrafRouta.Points.Add(tocka);
                if (i < this.BestFoundPointer-1)
                {
                    tocka.X = 50 + BestRouteGraf[i + 1, 1] * sirinaFaktor;
                }
                else
                {
                    tocka.X = 50+this.time_ciklus * sirinaFaktor;
                    optimum.Stroke = new SolidColorBrush(Colors.Blue);
                    optimum.StrokeThickness = 2;
                    optimum.Opacity = 1;
                    optimum.X1 = 50;
                    optimum.X2 = 50+NC_max*sirinaFaktor;
                    optimum.Y1 = 181 - (BestRouteGraf[i, 0] * visinaFaktor);
                    optimum.Y2 = optimum.Y1;
                }
                GrafRouta.Points.Add(tocka);
            }
            this.GrafArea.Children.Clear();
            this.GrafArea.Children.Add(optimum);
            this.GrafArea.Children.Add(GrafRouta);
            this.BrojeviDuljineGraf(170/(float)visinaFaktor, (float)BestRouteGraf[i-1,0], (float)optimum.Y1);
            return;
        }

        /// <summary>
        /// funkcija kojom Page prima vrijednosti koje opisuju graf routa
        /// </summary>
        /// <param name="maxVisina">visina grafa od koje se počne crtati</param>
        /// <param name="optimum">visina crte na kojoj je optimum</param>
        private void BrojeviDuljineGraf(float maxVisina, float optimum, float visina)
        {
            // max
            TextBlock max = new TextBlock();
            max.Text = (maxVisina).ToString();
            max.FontSize = 10;
            max.Opacity = 1;
            max.Foreground = new SolidColorBrush(Colors.Black);
            max.SetValue(Canvas.LeftProperty, 1.0);
            max.SetValue(Canvas.TopProperty, 1.0);
            this.GrafArea.Children.Add(max);
            // min
            TextBlock min = new TextBlock();
            min.Text = (0).ToString();
            min.FontSize = 10;
            min.Opacity = 1;
            min.Foreground = new SolidColorBrush(Colors.Black);
            min.SetValue(Canvas.LeftProperty, 21.0);
            min.SetValue(Canvas.TopProperty, 167.0);
            this.GrafArea.Children.Add(min);
            // opt
            TextBlock opt = new TextBlock();
            opt.Text = (optimum).ToString();
            opt.FontSize = 10;
            opt.Opacity = 1;
            opt.Foreground = new SolidColorBrush(Colors.Black);
            opt.SetValue(Canvas.LeftProperty, 1.0);
            opt.SetValue(Canvas.TopProperty, (double)(visina - 7));
            this.GrafArea.Children.Add(opt);
            return;
        }

        /// <summary>
        /// postavlja područje za crtanje, izcrtava gradove iz liste
        /// </summary>
        private void InitDraw()
        {
            this.DrawArea.Children.Clear();
            for (int i = 0; i < this.brojGradova + 1; i++)
            //foreach (grad g in gradovi)
            {
                Grad g = this.gradovi[i];
                if ((g.get_x() != 0) || (g.get_y() != 0))
                {
                    Ellipse circle = new Ellipse();
                    circle.Width = 4;
                    circle.Height = 4;
                    circle.Stroke = new SolidColorBrush(Colors.Blue);
                    circle.Fill = new SolidColorBrush(Colors.Blue);
                    circle.StrokeThickness = 1;
                    circle.SetValue(Canvas.LeftProperty, g.get_x() * this.faktorSirine);
                    circle.SetValue(Canvas.TopProperty, g.get_y() * this.faktorVisine);
                    if (this.TipProblema == tipProblema.VRP)
                    {
                        if (g == gradovi[1])
                        {
                            circle.Width = 9;
                            circle.Height = 9;
                            circle.SetValue(Canvas.LeftProperty, g.get_x() * this.faktorSirine - 2);
                            circle.SetValue(Canvas.TopProperty, g.get_y() * this.faktorVisine - 2);
                            circle.Fill = new SolidColorBrush(Colors.Yellow);
                        }
                    }
                    this.DrawArea.Children.Add(circle);
                };
            }
            return;
        }
                
        #endregion

        #endregion

        #region Properties

        /// <summary>
        /// Singleton obrazac za koloniju mrava
        /// </summary>
        public static AntColony Instance
        {
            get
            {
                if (AntColony.instance == null)
                {
                    AntColony.instance = new AntColony();
                }
                return AntColony.instance;
            }
        }

        #endregion

    }
}
