﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Text;
using System.Collections.Generic;
using SL_ACO;

namespace SL_ACO.ACO
{
    internal class Mrav
    {
        #region Fields

        internal int[] Tabu_lista;        //Tabu lista popisuje sve dosadašnje gradove
        internal int GradovaJeProsao;     //broj gradova obiđenih
        private int pointer;              //pokazuje na mjesto u tabu listi, treba biti isti kao steptimer u coloniji
        private int brojGradova;             //broj gradova u problemu
        private int LastHome;             //pointer u trenutku kad je bio u pocetnom gradu zadnji put
        private int brojViskaGradovaPriVRP; //sinkronizacija vozila u VRP-u (nejednolik broj gradova po vozilu)

        #endregion

        #region Methods

        /// <summary>
        /// pokreće mrava na iduće mjesto
        /// </summary>
        internal void nextStep()
        {
            int i;
            double[] p = new double[brojGradova + 2]; //array za vjerojatnost
            //Provjeri dal je doslo do nekakvog buga
            if (this.Tabu_lista[this.pointer] == 0)
            {
                this.pointer = 0;
            };
            // VRP vraćanje u početni grad
            if (AntColony.Instance.Get_tip() == tipProblema.VRP)
            {
                this.VratiSeUPocetniVRP();
            }

            // RAČUN VJEROJATNOSTI KORAKA
            #region zbrajanje vjerojatnosti
            int contains = 0;
            double suma = 0;        //zbraja sumu p[i] kako bi kasnije bili podijeljeni s njom za vjerojatnost 0<x<1
            for (i = 1; i < brojGradova + 1; i++)
            {
                //provjerava dali je grad i već posjećen
                contains = 0;
                foreach (int lok in Tabu_lista)
                {
                    if (i == lok)
                    {
                        contains = 1;
                    };
                };

                //ukoliko grad jos nije posjećen računaj vjerojatnost da će mrav ići tamo
                if (contains == 0)
                {
                    //vidljivost = 1 / udaljenost(i,j)
                    double Vidljivost = 1.0 / (Math.Sqrt(
                        Math.Pow(AntColony.Instance.gradovi[this.Tabu_lista[this.pointer]].get_x() - AntColony.Instance.gradovi[i].get_x(), 2) +
                        Math.Pow(AntColony.Instance.gradovi[this.Tabu_lista[this.pointer]].get_y() - AntColony.Instance.gradovi[i].get_y(), 2)));
                    p[i] = (double)(Math.Pow(AntColony.Instance.Trail[this.Tabu_lista[this.pointer], i], AntColony.Instance.Get_alpha()) *
                        Math.Pow(Vidljivost, AntColony.Instance.Get_beta()));
                    suma += p[i];
                }
                else
                {
                    p[i] = 0;
                };
            }
            #endregion

            //ukoliko je nadjen barem jedan grad u kojeg moze ići
            // odi u neki od njih, inace se vrati u pocetni grad
            #region Odi U Grad
            if (suma > 0)
            {
                //odluka za grad prema random broju
                int j = 0;
                double sum2=0;
                //podijeliti sve p[i] sa sumom da se dobije vjerojatnost 0<p<1 
                for (i = 1; i < brojGradova + 1; i++)
                {
                    p[i] = p[i] / suma;
                    sum2 += p[i];
                }
                //nađi random broj koji će biti slučajna odluka kamo ići 0<x<1
                double randomBroj = (double)AntColony.Instance.randomizer.NextDouble();
                while ((randomBroj == 0) || (randomBroj == 1))
                {
                    randomBroj = (double)AntColony.Instance.randomizer.NextDouble();
                };
                while (randomBroj > sum2)
                {
                    randomBroj = randomBroj - sum2; //sigurnosna adaptacija randomizera, zbog okvira float i double pogreske
                }
                //nađi redni broj grada u kojeg ide mrav prema random - vjerojatnost za taj grad
                while (randomBroj > 0)
                {
                    j++;
                    randomBroj -= p[j];
                }

                //odi u taj grad
                this.pointer++;
                this.Tabu_lista[this.pointer] = j;
                this.GradovaJeProsao ++;
                return;
            }
            else
            {
                //ako su svi gradovi obiđeni vrati se u svoj početni grad ako jos nisi tamo
                if (this.Tabu_lista[this.pointer] != this.Tabu_lista[0])
                {
                    this.pointer++;
                    this.Tabu_lista[this.pointer] = this.Tabu_lista[0];
                    this.GradovaJeProsao++;
                }
                return;
            }
            #endregion
        }

        /// <summary>
        /// pregledava se u slucaju VRPa prije svakog koraka, ako je obisao nGradova/brojVozila
        /// gradova onda se vrati u pocetni, pa nastavi sa korakom
        /// </summary>
        private void VratiSeUPocetniVRP()
        {
            //racun pocetnog ViskaGradova
            if (this.pointer == 0)
            {
                this.brojViskaGradovaPriVRP = (this.brojGradova - 1) % AntColony.Instance.Get_brojVozila();
            }
            //provjeri da li treba prebacit u novo vozilo, ako da odi u pocetni grad
            int brojkica = (this.brojGradova - 1 - this.brojViskaGradovaPriVRP) / AntColony.Instance.Get_brojVozila();
            if ((this.GradovaJeProsao - 1 - this.LastHome) >= (brojkica))
            {
                if ((this.brojViskaGradovaPriVRP > 0) && ((this.GradovaJeProsao - 1 - this.LastHome) < (brojkica + 1)))
                {
                    this.brojViskaGradovaPriVRP -= 1;
                }
                else
                {
                    this.pointer++;
                    this.Tabu_lista[this.pointer] = this.Tabu_lista[0];
                    this.GradovaJeProsao++;
                    this.LastHome = this.pointer;
                }
            }
        }

        /// <summary>
        /// konstruktor mrava
        /// </summary>
        /// <param name="n">prima broj koliko gradova ima u problemu (za rezervaciju memorije)</param>
        internal Mrav(int n, int pocetniGrad)
        {
            this.brojViskaGradovaPriVRP = 0;
            this.LastHome = 0;
            this.Tabu_lista = new int[2*n];
            this.pointer = 0;
            this.GradovaJeProsao = 1;
            this.brojGradova = n;
            this.Tabu_lista[this.pointer] = pocetniGrad;
        }

        /// <summary>
        /// ukupna duljina koju je mrav prešao u ovom ciklusu,
        /// automatski poziva Postavi Trail, jer ovisi o pređenoj duljini
        /// </summary>
        /// <returns>duljina</returns>
        internal double DuljinaPuta()
        {
            int i=0;
            int j=0;
            double duljina=0;
            for (i = 0; i < 2 * brojGradova - 1; i++)
            {
                j = i + 1;
                if ((this.Tabu_lista[i] > 0) && (this.Tabu_lista[j] > 0))
                {
                    //pitagorin poucak
                    duljina += Math.Sqrt(
                        Math.Pow(AntColony.Instance.gradovi[Tabu_lista[i]].get_x() - AntColony.Instance.gradovi[Tabu_lista[j]].get_x(), 2) +
                        Math.Pow(AntColony.Instance.gradovi[Tabu_lista[i]].get_y() - AntColony.Instance.gradovi[Tabu_lista[j]].get_y(), 2));
                }
            }
            this.PostaviTrail(duljina);
            return duljina;
        }

        /// <summary>
        /// postavlja trag feromona na sve puteve na kojima je bio
        /// sto veca duljina puta koju je mrav presao to je manji njegov trail
        /// po formuli 10×nGradova/duljina
        /// </summary>
        /// <param name="duljina">duljina puta koju je mrav presao u ovom ciklusu</param>
        private void PostaviTrail(double duljina)
        {
            int i, j;
            for (i = 1; i < 2 * this.brojGradova - 1; i++)
            {
                j = i + 1;
                if ((Tabu_lista[i] != 0) && (Tabu_lista[j] != 0))
                {
                    AntColony.Instance.Trail[Tabu_lista[i], Tabu_lista[j]] += (float)((10 * brojGradova) / (float)duljina);
                    AntColony.Instance.Trail[Tabu_lista[j], Tabu_lista[i]] += (float)((10 * brojGradova) / (float)duljina);
                }
            }
        }

        /// <summary>
        /// ocisti mrava / postavi ga kao da je nov.
        /// </summary>
        internal void Cleanup()
        {
            this.LastHome = 0;
            int i;
            for (i = 1; i < 2 * this.brojGradova; i++)
            {
                this.Tabu_lista[i] = 0;
            };
            this.pointer = 0;
            this.brojViskaGradovaPriVRP = 0;
            this.GradovaJeProsao = 1;
            return;
        }

        #endregion

    }
}
