﻿using System;
using System.Net;
using System.Text;
using System.Security;
using System.Collections.Generic;
using System.Windows.Media.Imaging;
using System.Windows.Browser;
using System.Windows.Resources;
using System.Windows.Controls;
using SL_ACO;
using SL_ACO.ACO;
using System.IO;
using System.Windows;

namespace SL_ACO.ACO
{
    internal class Ulaz
    {
        #region Fields

        internal tipProblema Tip;     //enum tip da li je problem TSP ili VRP
        internal List<Grad> lista;    //lista gradova koja se stvori
        internal int brojVozila;      //broj vozila koji se smiju koristiti, u slučaju VRP
        internal int brojGradova;     //broj stvorenih gradova

        #endregion

        #region Methods

        /// <summary>
        /// Overloadan konstruktor za učitavanje ulazne datoteke za stvaranje problema
        /// </summary>
        /// <param name="ulazTrue">0 = random, 1 = load file, 2= load krugovi, 3= load kocke, 4 = ACO</param>
        internal Ulaz(int ulazType)
        {
            //ograničavanje Stored problema na 3 (2,3,4)
            //0 i 1 ne dira (random i load verzije)
            while ((ulazType > 4)||(ulazType<0))
            {
                ulazType = 2 + ulazType % 3; 
            }
            switch (ulazType)
            {
                case 0: 
                    // random
                    this.RandomGenerator();
                    AntColony.Instance.Set_m(this.lista.Count);
                    return;
                case 1:
                    // cita iz ulazniProblem
                    AntColony.Instance.Set_m(1);
                    this.Citac();
                    if (AntColony.Instance.Get_m() == 1)
                    {
                        AntColony.Instance.Set_m(this.lista.Count);
                    }
                    return;
                case 2: 
                    // krugovi
                    this.StvoriKrugove(2 + AntColony.Instance.randomizer.Next(7));
                    AntColony.Instance.Set_m(this.lista.Count);
                    return;
                case 3: 
                    // grid n×n
                    this.StvoriCube(4 + AntColony.Instance.randomizer.Next(10));
                    AntColony.Instance.Set_m(this.lista.Count);
                    return;
                case 4: 
                    // napise ACO
                    this.StvoriACO();
                    AntColony.Instance.Set_m(this.lista.Count);
                    return;
                default:
                    return;
            }
        }

        /// <summary>
        /// citac za ulaznu datoteku, nudi prozor otvaranja datoteke
        /// nakon toga ju parsira i stvori problem ukoliko je odgovarajućeg tipa (TSP ili VRP)
        /// </summary>
        private void Citac()
        {
            //otvaranje datoteke
            this.lista = new List<Grad>();
            OpenFileDialog datoteka = new OpenFileDialog();
            datoteka.Multiselect = false;
            datoteka.ShowDialog();
            if (datoteka.File == null)
            {
                return;
            }
            FileStream citac = datoteka.File.OpenRead();
            //citanje datoteke
            byte[] input = new byte[5000];
            citac.Read(input, 0, 4999);

            try
            {
                //parsiranje procitanog
                #region Tip problema i broj vozila
                int i = 0;
                string Ulazni = "";
                for (i = 0; i < 3; i++)
                {
                    char c = (char)input[i];
                    Ulazni += c.ToString();
                }
                switch (Ulazni)
                {
                    case "TSP":
                        this.Tip = tipProblema.TSP;
                        this.brojVozila = 4;
                        break;
                    case "VRP":
                        this.Tip = tipProblema.VRP;
                        this.brojVozila = 4;
                        break;
                    default:
                        this.Tip = tipProblema.error;
                        MessageBox.Show("Nepoznat tip problema u datoteci! poznati samo TSP i VRP");
                        return;
                }//end switch
                #endregion
                #region stvaranje Gradova i parametara
                i = 0;
                while (input[i] != 0)
                {
                    Ulazni = "";
                    //preskoci prvi red (definicija programa)
                    if (i == 0)
                    {
                        while ((char)input[i] != '\n')
                        {
                            i++;
                        }
                    }
                    //pobrini se da dodjes na pocetak novog nepraznog reda
                    while (((char)input[i] == '\n')||((char)input[i] == '\r'))
                    {
                        i++;
                    }
                    // Parsiraj red
                    switch ((char)input[i])
                    {
                        case '#': //komentari - preskaci
                            while ((char)input[i] != '\n')
                            {
                                i++;
                            }
                            break;
                        case '!': //parametar optimizacije - ukucaj
                            i++;
                            string kod = "";
                            while (((char)input[i] != ' ') && (input[i] != 0) && ((char)input[i] != '\r'))
                            {
                                char c = (char)input[i];
                                kod += c.ToString();
                                i++;
                            }
                            i++; //preskoči razmak
                            while (((char)input[i] != '\n') && (input[i] != 0) && ((char)input[i] != '\r'))
                            {
                                char c = (char)input[i];
                                Ulazni += c.ToString();
                                i++;
                            }
                            switch (kod) //postavi određeni parametar
                            {
                                case "mrav":
                                    AntColony.Instance.Set_m(int.Parse(Ulazni));
                                    break;
                                case "NC":
                                    AntColony.Instance.Set_NC(int.Parse(Ulazni));
                                    break;
                                case "alpha":
                                    AntColony.Instance.Set_alpha(float.Parse(Ulazni));
                                    break;
                                case "beta":
                                    AntColony.Instance.Set_beta(float.Parse(Ulazni));
                                    break;
                                case "rho":
                                    AntColony.Instance.Set_rho(float.Parse(Ulazni));
                                    break;
                                case "vozila":
                                    AntColony.Instance.Set_vozila(int.Parse(Ulazni));
                                    this.brojVozila = int.Parse(Ulazni);
                                    break;
                                default:
                                    //nepoznat parametar
                                    break;
                            }//end parametar switch
                            break;
                        default: //grad - dodaj grad
                            while (((char)input[i] != '\n') && (input[i] != 0) && ((char)input[i] != '\r'))
                            {
                                char c = (char)input[i];
                                Ulazni += c.ToString();
                                i++;
                            }
                            if (Ulazni.Length>1)
                            {
                                string[] brojeviString = new string[2];
                                brojeviString = Ulazni.Split(' ');
                                double[] brojevi = new double[2];
                                brojevi[0] = double.Parse(brojeviString[0]);
                                brojevi[1] = double.Parse(brojeviString[1]);
                                this.lista.Add(new Grad(brojevi[0], brojevi[1]));
                            }
                            break;
                    }//end switch
                }//end Parser
                #endregion
            }
            catch
            {
                //neispravna/nepoznata datoteka
                MessageBox.Show("Neispravna datoteka ili zapis u njoj");
            }
            citac.Close();
            this.brojGradova = lista.Count;
            return;
        }

        /// <summary>
        /// random stvaranje ulaza
        /// </summary>
        private void RandomGenerator()
        {
            Random stroj = new Random();
            double x = 0;
            double y = 0;
            int i = 0;
            int n = 30 + stroj.Next(100);
            this.brojGradova = n;
            this.lista = new List<Grad>();
            for (i = 1; i < n + 1; i++)
            {
                x = stroj.NextDouble();
                y = stroj.NextDouble();
                this.lista.Add(new Grad(x, y));
            }
            if (stroj.Next(2) == 0)
            {
                this.Tip = tipProblema.TSP;
            }
            else
            {
                this.Tip = tipProblema.VRP;
            }
            this.brojVozila = 2 + stroj.Next(7);
            return;
        }

        /// <summary>
        /// stvori gradove u obliku slova ACO
        /// </summary>
        private void StvoriACO()
        {
            Tip = tipProblema.TSP;
            brojVozila = 3;
            lista = new List<Grad>();
            // A
            lista.Add(new Grad(150, 150));
            lista.Add(new Grad(110, 350));
            lista.Add(new Grad(190, 350));
            lista.Add(new Grad(120, 300));
            lista.Add(new Grad(180, 300));
            lista.Add(new Grad(170, 250));
            lista.Add(new Grad(160, 200));
            lista.Add(new Grad(130, 250));
            lista.Add(new Grad(140, 200));
            lista.Add(new Grad(142, 250));
            lista.Add(new Grad(158, 250));
            // C
            lista.Add(new Grad(240, 190));
            lista.Add(new Grad(230, 250));
            lista.Add(new Grad(240, 310));
            lista.Add(new Grad(260, 350));
            lista.Add(new Grad(260, 150));
            lista.Add(new Grad(300, 350));
            lista.Add(new Grad(300, 150));
            lista.Add(new Grad(340, 320));
            lista.Add(new Grad(340, 180));
            // O
            lista.Add(new Grad(410, 190));
            lista.Add(new Grad(400, 250));
            lista.Add(new Grad(410, 310));
            lista.Add(new Grad(440, 350));
            lista.Add(new Grad(440, 150));
            lista.Add(new Grad(480, 350));
            lista.Add(new Grad(480, 150));
            lista.Add(new Grad(510, 310));
            lista.Add(new Grad(520, 250));
            lista.Add(new Grad(510, 190));
            brojGradova = lista.Count;
            return;
        }

        /// <summary>
        /// stvara grid gradova n×n za n=brojResetki 
        /// </summary>
        /// <param name="brojResetki">broj redova/stupaca gradova</param>
        private void StvoriCube(int brojResetki)
        {
            Tip = tipProblema.TSP;
            brojVozila = 4;
            lista = new List<Grad>();
            int i = 0;
            int j=0;
            for (i = 0; i < brojResetki; i++)
            {
                for (j = 0; j < brojResetki; j++)
                {
                    lista.Add(new Grad(10 + i * 50, 10 + j * 50));
                }
            }
            brojGradova = lista.Count;
            return;
        }

        /// <summary>
        /// stvara problem 3 koncentricna kruga, 
        /// broj gradova = 12*(brojGradovaUKvadrantu)
        /// </summary>
        /// <param name="brojGradovaUKvadrantu">koliko gradova da bude u svakom kvadrantu</param>
        private void StvoriKrugove(int brojGradovaUKvadrantu)
        {
            Tip = tipProblema.TSP;
            brojVozila = 3;
            lista = new List<Grad>();
            int j = 0;
            double X_ordinata = 0;
            double Y_ordinata = 0;
            for (j = 0; j < brojGradovaUKvadrantu+1; j++)
            {
                X_ordinata = 220 * (Math.Cos(j * 3.1412 / (brojGradovaUKvadrantu*2)));
                Y_ordinata = 220 * (Math.Sin(j * 3.1412 / (brojGradovaUKvadrantu*2)));
                lista.Add(new Grad(275 + X_ordinata, 275 + Y_ordinata));
                lista.Add(new Grad(275 + X_ordinata / 3, 275 + Y_ordinata / 3));
                lista.Add(new Grad(275 + X_ordinata * 2 / 3, 275 + Y_ordinata * 2 / 3));
                if (j < brojGradovaUKvadrantu)
                {
                    lista.Add(new Grad(275 - X_ordinata, 275 + Y_ordinata));
                    lista.Add(new Grad(275 - X_ordinata / 3, 275 + Y_ordinata / 3));
                    lista.Add(new Grad(275 - X_ordinata * 2 / 3, 275 + Y_ordinata * 2 / 3));
                }
                if (j > 0)
                {
                    if (j < brojGradovaUKvadrantu)
                    {
                        lista.Add(new Grad(275 + X_ordinata, 275 - Y_ordinata));
                        lista.Add(new Grad(275 + X_ordinata / 3, 275 - Y_ordinata / 3));
                        lista.Add(new Grad(275 + X_ordinata * 2 / 3, 275 - Y_ordinata * 2 / 3));
                    }
                    lista.Add(new Grad(275 - X_ordinata, 275 - Y_ordinata));
                    lista.Add(new Grad(275 - X_ordinata / 3, 275 - Y_ordinata / 3));
                    lista.Add(new Grad(275 - X_ordinata * 2 / 3, 275 - Y_ordinata * 2 / 3));
                }
            }
            brojGradova = lista.Count;
            return;
        }

        #endregion

    }
}
