﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Net;
using System.Windows;
using System.Windows.Media;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Browser;
using System.IO;
using SL_ACO;
using SL_ACO.ACO;

namespace SL_ACO
{
    public partial class Page : UserControl
    {
        #region Initial

        /// <summary>
        /// otvaranje stranice
        /// </summary>
        public Page()
        {
            InitializeComponent();
            AntColony.Instance.JaviPage(this,this.Graf_Patha, this.Skica);
            this.UpdateStats();
        }

        #endregion

        #region MouseCanvas

        /// <summary>
        /// dodaje grad na mjesto gdje se kliknulo na canvasu
        /// </summary>
        /// <param name="sender">pozivatelj</param>
        /// <param name="e">mjesto gdje je bio miš u kliku</param>
        private void Skica_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            Point mjesto = e.GetPosition(this.Skica);
            AntColony.Instance.AddCity((float) mjesto.X, (float) mjesto.Y);
            this.UpdateStats();
        }

        #endregion

        #region Button Clicks

        /// <summary>
        /// klick na gumb reset
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Reset_Click(object sender, RoutedEventArgs e)
        {
            AntColony.Instance.Reset();
        }

        /// <summary>
        /// klick na gumb start, pokrece optimizaciju
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Start_Click(object sender, RoutedEventArgs e)
        {
            AntColony.Instance.Start();
        }

        /// <summary>
        /// klik na gumb step, pokrece algoritam za jedan step
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Step_Click(object sender, RoutedEventArgs e)
        {
            AntColony.Instance.Step();
        }

        /// <summary>
        /// klick na gumb read; inicijalizira Ant Colony i postavi grad i mrave
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Read_Click(object sender, RoutedEventArgs e)
        {
            AntColony.Instance.Read(1);
            this.UpdateStats();
        }

        /// <summary>
        /// klick na gumb Clean, daje prazni list gradova
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Clean_Click(object sender, RoutedEventArgs e)
        {
            AntColony.Instance.Read(2);
            AntColony.Instance.Set_Tip(0);
            this.UpdateStats();
        }

        /// <summary>
        /// Clean za VRP, prazni list gradova, prvi je pocetni
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void VRP_Clean_Click(object sender, RoutedEventArgs e)
        {
            AntColony.Instance.Read(2);
            AntColony.Instance.Set_Tip(tipProblema.VRP);
            this.UpdateStats();
        }

        /// <summary>
        /// mijenja tip problema, ako postane VRP onda prvi grad iz TSP postane ishodište
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Change_Tip_Click(object sender, RoutedEventArgs e)
        {
            AntColony.Instance.Set_Tip(1 - AntColony.Instance.Get_tip());
            AntColony.Instance.Reset();
            this.UpdateStats();
        }

        /// <summary>
        /// ucita jedan od sejvanih problema
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Stored_Click(object sender, RoutedEventArgs e)
        {
            AntColony.Instance.Read(4);
            this.UpdateStats();
        }

        /// <summary>
        /// klick na gumb Load, učitava iz datoteke problem
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Load_Click(object sender, RoutedEventArgs e)
        {
            AntColony.Instance.Read(3);
            this.UpdateStats();
        }

        /// <summary>
        /// proba sejvat problem na klijent na targetan path
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Save_Click(object sender, RoutedEventArgs e)
        {
            AntColony.Instance.Save(this.FileNamebox.Text+".txt");
            this.UpdateStats();
        }

        #endregion

        #region Methods

        /// <summary>
        /// obnavlja prikazane vrijednosti u interface-u prema vrijednostima u koloniji
        /// </summary>
        private void UpdateStats()
        {
            int vozila = AntColony.Instance.Get_brojVozila();
            this.vozila_box.Text = string.Copy(vozila.ToString());
            float alpha = AntColony.Instance.Get_alpha();
            this.alpha_box.Text = string.Copy(alpha.ToString());
            float beta = AntColony.Instance.Get_beta();
            this.beta_box.Text = string.Copy(beta.ToString());
            float rho = AntColony.Instance.Get_rho();
            this.rho_box.Text = string.Copy(rho.ToString());
            int m = AntColony.Instance.Get_m();
            this.m_box.Text = string.Copy(m.ToString());
            int NC = AntColony.Instance.Get_NC();
            this.NC_box.Text = string.Copy(NC.ToString());
            int n = AntColony.Instance.Get_n();
            this.BrojGradova_box.Text = string.Copy(n.ToString());
            tipProblema tip = AntColony.Instance.Get_tip();
            if (tip == tipProblema.VRP)
            {
                this.tipProblema_box.Text = "VRP";
            }
            else if (tip == tipProblema.TSP)
            {
                this.tipProblema_box.Text = "TSP";
            };
        }

        /// <summary>
        /// funkcija kojom prima izmjerene vrijednosti iz kolonije kad završi
        /// </summary>
        /// <param name="CiklusCounter">broj ciklusa koje je kolonija izvrtila</param>
        /// <param name="BestRoute">duljina najboljeg puta nadjenog</param>
        /// <param name="BestFoundAt">ciklus u kojem je nadjena najbolja ruta</param>
        public void Novosti(int CiklusCounter, double BestRoute, int BestFoundAt, string Time)
        {
            this.ciklus_box.Text = string.Copy(CiklusCounter.ToString());
            this.best_box.Text = string.Copy(BestRoute.ToString());
            this.found_box.Text = string.Copy(BestFoundAt.ToString());
            this.Time_box.Text = string.Copy(Time);
        }

        #endregion

        #region Box_changes

        /// <summary>
        /// event promijene vrijednosti alphe
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void alpha_box_TextChanged(object sender, TextChangedEventArgs e)
        {
            float broj = 0;
            int i = 0;
            int dec = 0;
            string read;
            read = string.Copy(this.alpha_box.Text.ToString()) + '\0';
            while (read[i] != '\0')
            {
                if (read[i] == ',')
                {
                    dec = 1;
                }
                else
                {
                    broj = 10 * broj + Math.Abs(read[i]-'0');
                }
                dec *= 10;
                i++;
            }
            dec = dec / 10;
            if (dec == 1) { return; };
            if (dec == 0) { dec = 1; };
            broj = broj / dec;
            AntColony.Instance.Set_alpha(broj);
            this.UpdateStats();
        }

        /// <summary>
        /// event promijene vrijednosti m
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void m_box_TextChanged(object sender, TextChangedEventArgs e)
        {
            int broj = 0;
            int i = 0;
            int dec = 0;
            string read;
            read = string.Copy(this.m_box.Text.ToString()) + '\0';
            while (read[i] != '\0')
            {
                if (read[i] == ',')
                {
                    dec = 1;
                }
                else
                {
                    broj = 10 * broj + Math.Abs(read[i]-'0');
                }
                dec *= 10;
                i++;
            }
            dec = dec / 10;
            if (dec == 1) { return; };
            if (dec == 0) { dec = 1; };
            broj = broj / dec;
            AntColony.Instance.Set_m(broj);
            this.UpdateStats();
        }

        /// <summary>
        /// event promijene vrijednosti beta
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void beta_box_TextChanged(object sender, TextChangedEventArgs e)
        {
            float broj = 0;
            int i = 0;
            int dec = 0;
            string read;
            read = string.Copy(this.beta_box.Text.ToString()) + '\0';
            while (read[i] != '\0')
            {
                if (read[i] == ',')
                {
                    dec = 1;
                }
                else
                {
                    broj = 10 * broj + Math.Abs(read[i]-'0');
                }
                dec *= 10;
                i++;
            }
            dec = dec / 10;
            if (dec == 1) { return; };
            if (dec == 0) { dec = 1; };
            broj = broj / dec;
            AntColony.Instance.Set_beta(broj);
            this.UpdateStats();
        }

        /// <summary>
        /// event promijene vrijednosti rho
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void rho_box_TextChanged(object sender, TextChangedEventArgs e)
        {
            float broj = 0;
            int i = 0;
            int dec = 0;
            string read;
            read = string.Copy(this.rho_box.Text.ToString()) + '\0';
            while (read[i] != '\0')
            {
                if (read[i] == ',')
                {
                    dec = 1;
                }
                else
                {
                    broj = 10 * broj + Math.Abs(read[i]-'0');
                }
                dec *= 10;
                i++;
            }
            dec = dec / 10;
            if (dec == 1) { return; };
            if (dec == 0) { dec = 1; };
            broj = broj / dec;
            AntColony.Instance.Set_rho(broj);
            this.UpdateStats();
        }

        /// <summary>
        /// event promijene vrijednosti NC
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void NC_box_TextChanged(object sender, TextChangedEventArgs e)
        {
            int broj = 0;
            int i = 0;
            int dec = 0;
            string read;
            read = string.Copy(this.NC_box.Text.ToString()) + '\0';
            while (read[i] != '\0')
            {
                if (read[i] == ',')
                {
                    dec = 1;
                }
                else
                {
                    broj = 10 * broj + Math.Abs(read[i]-'0');
                }
                dec *= 10;
                i++;
            }
            dec = dec / 10;
            if (dec == 1) { return; };
            if (dec == 0) { dec = 1; };
            broj = broj / dec;
            AntColony.Instance.Set_NC(broj);
            this.UpdateStats();
        }

        /// <summary>
        /// event promjene texta u boxu vozila
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void vozila_box_TextChanged(object sender, TextChangedEventArgs e)
        {
            int broj = int.Parse(this.vozila_box.Text.ToString());
            AntColony.Instance.Set_vozila(broj);
            this.UpdateStats();
        }


        #endregion

    }
}
